#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .YaDiskObject import YaDiskObject
from .disk import UserObject
from ..common import typed_list, yandex_date

__all__ = ["CommentIDsObject", "EXIFObject", "FilesResourceListObject",
           "LastUploadedResourceListObject", "LinkObject", "OperationLinkObject",
           "PublicResourcesListObject", "ResourceListObject", "ResourceObject",
           "ResourceUploadLinkObject", "ShareInfoObject", "PublicResourceObject",
           "PublicResourceListObject", "TrashResourceObject", "TrashResourceListObject"]

class CommentIDsObject(YaDiskObject):
    """
        Comment IDs object.

        :param comment_ids: `dict` or `None`

        :ivar private_resource: `str`, comment ID for private resources
        :ivar public_resource: `str`, comment ID for public resources
    """

    def __init__(self, comment_ids=None):
        YaDiskObject.__init__(self, {"private_resource": str,
                                     "public_resource":  str})

        self.import_fields(comment_ids)

class EXIFObject(YaDiskObject):
    """
        EXIF metadata object.

        :param exif: `dict` or `None`

        :ivar date_time: :any:`datetime.datetime`, capture date
    """

    def __init__(self, exif=None):
        YaDiskObject.__init__(self, {"date_time": yandex_date})

        self.import_fields(exif)

class FilesResourceListObject(YaDiskObject):
    """
        Flat list of files.

        :param files_resource_list: `dict` or `None`

        :ivar items: `list`, flat list of files (:any:`ResourceObject`)
        :ivar limit: `int`, maximum number of elements in the list
        :ivar offset: `int`, offset from the beginning of the list
    """

    def __init__(self, files_resource_list=None):
        YaDiskObject.__init__(self, {"items":  typed_list(ResourceObject),
                                     "limit":  int,
                                     "offset": int})

        self.import_fields(files_resource_list)

class LastUploadedResourceListObject(YaDiskObject):
    """
        List of last uploaded resources.

        :param last_uploaded_resources_list: `dict` or `None`

        :ivar items: `list`, list of resources (:any:`ResourceObject`)
        :ivar limit: `int`, maximum number of elements in the list
    """

    def __init__(self, last_uploaded_resources_list=None):
        YaDiskObject.__init__(self, {"items":  typed_list(ResourceObject),
                                     "limit":  int})
        self.import_fields(last_uploaded_resources_list)

class LinkObject(YaDiskObject):
    """
        Link object.

        :param link: `dict` or `None`

        :ivar href: `str`, link URL
        :ivar method: `str`, HTTP method
        :ivar templated: `bool`, tells whether the URL is templated
    """

    def __init__(self, link=None):
        YaDiskObject.__init__(self, {"href":      str,
                                     "method":    str,
                                     "templated": bool})

        self.import_fields(link)

class OperationLinkObject(LinkObject):
    """
        Operation link object.

        :param link: `dict` or `None`

        :ivar href: `str`, link URL
        :ivar method: `str`, HTTP method
        :ivar templated: `bool`, tells whether the URL is templated
    """

    pass

class PublicResourcesListObject(YaDiskObject):
    """
        List of public resources.

        :param public_resources_list: `dict` or `None`

        :ivar items: `list`, list of public resources (:any:`PublicResourceObject`)
        :ivar type: `str`, resource type to filter by
        :ivar limit: `int`, maximum number of elements in the list
        :ivar offset: `int`, offset from the beginning of the list
    """

    def __init__(self, public_resources_list=None):
        YaDiskObject.__init__(self, {"items":  typed_list(PublicResourceObject),
                                     "type":   str,
                                     "limit":  int,
                                     "offset": int})

        self.import_fields(public_resources_list)

class ResourceObject(YaDiskObject):
    """
        Resource object.

        :param resource: `dict` or `None`

        :ivar size: `int`, file size
        :ivar public_key: `str`, public resource key
        :ivar sha256: `str`, SHA256 hash
        :ivar md5: `str`, MD5 hash
        :ivar embedded: :any:`ResourceListObject`, list of nested resources
        :ivar name: `str`, filename
        :ivar exif: :any:`EXIFObject`, EXIF metadata
        :ivar resource_id: `str`, resource ID
        :ivar custom_properties: `dict`, custom resource properties
        :ivar public_url: `str`, public URL
        :ivar share: :any:`ShareInfoObject`, shared folder information
        :ivar modified: :any:`datetime.datetime`, date of last modification
        :ivar created: :any:`datetime.datetime`, date of creation
        :ivar mime_type: `str`, MIME type
        :ivar path: `str`, path to the resource
        :ivar preview: `str`, file preview URL
        :ivar comment_ids: :any:`CommentIDsObject`, comment IDs
        :ivar type: `str`, type ("file" or "dir")
        :ivar media_type: `str`, file type as determined by Yandex.Disk
        :ivar revision: `int`, Yandex.Disk revision at the time of last modification
    """

    def __init__(self, resource=None):
        YaDiskObject.__init__(self, {"size":              int,
                                     "public_key":        str,
                                     "sha256":            str,
                                     "embedded":          ResourceListObject,
                                     "name":              str,
                                     "exif":              EXIFObject,
                                     "resource_id":       str,
                                     "custom_properties": dict,
                                     "public_url":        str,
                                     "share":             ShareInfoObject,
                                     "modified":          yandex_date,
                                     "created":           yandex_date,
                                     "mime_type":         str,
                                     "path":              str,
                                     "preview":           str,
                                     "comment_ids":       CommentIDsObject,
                                     "type":              str,
                                     "media_type":        str,
                                     "md5":               str,
                                     "revision":          int})
        self.set_alias("_embedded", "embedded")
        self.import_fields(resource)

class ResourceListObject(YaDiskObject):
    """
        List of resources.

        :param resource_list: `dict` or `None`

        :ivar sort: `str`, sort type
        :ivar items: `list`, list of resources (:any:`ResourceObject`)
        :ivar limit: `int`, maximum number of elements in the list
        :ivar offset: `int`, offset from the beginning of the list
        :ivar path: `str`, path to the directory that contains the elements of the list
        :ivar total: `int`, number of elements in the list
    """

    def __init__(self, resource_list=None):
        YaDiskObject.__init__(self, {"sort":   str,
                                     "items":  typed_list(ResourceObject),
                                     "limit":  int,
                                     "offset": int,
                                     "path":   str,
                                     "total":  int})
        self.import_fields(resource_list)

class ResourceUploadLinkObject(LinkObject):
    """
        Resource upload link.

        :param resource_upload_link: `dict` or `None`

        :ivar operation_id: `str`, ID of the upload operation
        :ivar href: `str`, link URL
        :ivar method: `str`, HTTP method
        :ivar templated: `bool`, tells whether the URL is templated
    """

    def __init__(self, resource_upload_link=None):
        LinkObject.__init__(self)
        self.set_field_type("operation_id", str)
        self.import_fields(resource_upload_link)

class ShareInfoObject(YaDiskObject):
    """
        Shared folder information object.

        :param share_info: `dict` or `None`

        :ivar is_root: `bool`, tells whether the folder is root
        :ivar is_owned: `bool`, tells whether the user is the owner of this directory
        :ivar rights: `str`, access rights
    """

    def __init__(self, share_info=None):
        YaDiskObject.__init__(self, {"is_root":  bool,
                                     "is_owned": bool,
                                     "rights":   str})
        self.import_fields(share_info)

class PublicResourceObject(ResourceObject):
    """
        Public resource object.

        :param resource: `dict` or `None`

        :ivar size: `int`, file size
        :ivar public_key: `str`, public resource key
        :ivar sha256: `str`, SHA256 hash
        :ivar md5: `str`, MD5 hash
        :ivar embedded: :any:`PublicResourceObject`, list of nested resources
        :ivar name: `str`, filename
        :ivar exif: :any:`EXIFObject`, EXIF metadata
        :ivar resource_id: `str`, resource ID
        :ivar custom_properties: `dict`, custom resource properties
        :ivar public_url: `str`, public URL
        :ivar share: :any:`ShareInfoObject`, shared folder information
        :ivar modified: :any:`datetime.datetime`, date of last modification
        :ivar created: :any:`datetime.datetime`, date of creation
        :ivar mime_type: `str`, MIME type
        :ivar path: `str`, path to the resource
        :ivar preview: `str`, file preview URL
        :ivar comment_ids: :any:`CommentIDsObject`, comment IDs
        :ivar type: `str`, type ("file" or "dir")
        :ivar media_type: `str`, file type as determined by Yandex.Disk
        :ivar revision: `int`, Yandex.Disk revision at the time of last modification
        :ivar view_count: `int`, number of times the public resource was viewed
        :ivar owner: :any:`UserObject`, owner of the public resource
    """

    def __init__(self, public_resource=None):
        ResourceObject.__init__(self)
        self.set_field_type("views_count", int)
        self.set_alias("view_count", "views_count")
        self.set_field_type("embedded", PublicResourceListObject)
        self.set_field_type("owner", UserObject)
        self.import_fields(public_resource)

class PublicResourceListObject(ResourceListObject):
    """
        List of public resources.

        :param public_resource_list: `dict` or `None`

        :ivar sort: `str`, sort type
        :ivar items: `list`, list of resources (:any:`ResourceObject`)
        :ivar limit: `int`, maximum number of elements in the list
        :ivar offset: `int`, offset from the beginning of the list
        :ivar path: `str`, path to the directory that contains the elements of the list
        :ivar total: `int`, number of elements in the list
        :ivar public_key: `str`, public key of the resource
    """

    def __init__(self, public_resource_list=None):
        ResourceListObject.__init__(self)
        self.set_field_type("public_key", str)
        self.set_field_type("items", typed_list(PublicResourceObject))
        self.import_fields(public_resource_list)

class TrashResourceObject(ResourceObject):
    """
        Trash resource object.

        :param trash_resource: `dict` or `None`

        :ivar size: `int`, file size
        :ivar public_key: `str`, public resource key
        :ivar sha256: `str`, SHA256 hash
        :ivar md5: `str`, MD5 hash
        :ivar embedded: :any:`TrashResourceListObject`, list of nested resources
        :ivar name: `str`, filename
        :ivar exif: :any:`EXIFObject`, EXIF metadata
        :ivar resource_id: `str`, resource ID
        :ivar custom_properties: `dict`, custom resource properties
        :ivar public_url: `str`, public URL
        :ivar share: :any:`ShareInfoObject`, shared folder information
        :ivar modified: :any:`datetime.datetime`, date of last modification
        :ivar created: :any:`datetime.datetime`, date of creation
        :ivar mime_type: `str`, MIME type
        :ivar path: `str`, path to the resource
        :ivar preview: `str`, file preview URL
        :ivar comment_ids: :any:`CommentIDsObject`, comment IDs
        :ivar type: `str`, type ("file" or "dir")
        :ivar media_type: `str`, file type as determined by Yandex.Disk
        :ivar revision: `int`, Yandex.Disk revision at the time of last modification
        :ivar origin_path: `str`, original path
        :ivar deleted: :any:`datetime.datetime`, date of deletion
    """

    def __init__(self, trash_resource=None):
        ResourceObject.__init__(self)
        self.set_field_type("embedded", TrashResourceListObject)
        self.set_field_type("origin_path", str)
        self.set_field_type("deleted", yandex_date)
        self.import_fields(trash_resource)

class TrashResourceListObject(ResourceListObject):
    """
        List of trash resources.

        :param trash_resource_list: `dict` or `None`

        :ivar sort: `str`, sort type
        :ivar items: `list`, list of resources (:any:`TrashResourceObject`)
        :ivar limit: `int`, maximum number of elements in the list
        :ivar offset: `int`, offset from the beginning of the list
        :ivar path: `str`, path to the directory that contains the elements of the list
        :ivar total: `int`, number of elements in the list
    """

    def __init__(self, trash_resource_list=None):
        ResourceListObject.__init__(self)
        self.set_field_type("items", typed_list(TrashResourceObject))
        self.import_fields(trash_resource_list)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .APIRequest import APIRequest
from ..objects import DiskInfoObject

__all__ = ["DiskInfoRequest"]

class DiskInfoRequest(APIRequest):
    """
        A request to get disk information.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param fields: list of keys to be included in the response

        :returns: :any:`DiskInfoObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk"
    method = "GET"

    def __init__(self, session, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"fields": fields}, *args, **kwargs)

    def process_args(self, fields):
        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return DiskInfoObject(js)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import datetime

__all__ = ["typed_list", "yandex_date", "is_operation_link"]

def typed_list(datatype):
    def list_factory(iterable=None):
        if iterable is None:
            return []

        return [datatype(i) for i in iterable]

    return list_factory

def yandex_date(string):
    return datetime.datetime.strptime(string[:-3] + string[-2:], "%Y-%m-%dT%H:%M:%S%z")

def is_operation_link(link):
    return link.startswith("https://cloud-api.yandex.net/v1/disk/operations/")



# -*- coding: utf-8 -*-

from collections import defaultdict
import time

import requests.exceptions

from .objects import ErrorObject
from .exceptions import *
from . import settings

__all__ = ["get_exception", "auto_retry"]

EXCEPTION_MAP = {400: defaultdict(lambda: BadRequestError,
                                  {"FieldValidationError": FieldValidationError}),
                 401: defaultdict(lambda: UnauthorizedError),
                 403: defaultdict(lambda: ForbiddenError),
                 404: defaultdict(lambda: NotFoundError,
                                  {"DiskNotFoundError": PathNotFoundError}),
                 406: defaultdict(lambda: NotAcceptableError),
                 409: defaultdict(lambda: ConflictError,
                                  {"DiskPathDoesntExistsError": ParentNotFoundError,
                                   "DiskPathPointsToExistentDirectoryError": DirectoryExistsError,
                                   "DiskResourceAlreadyExistsError": PathExistsError}),
                 415: defaultdict(lambda: UnsupportedMediaError),
                 423: defaultdict(lambda: LockedError,
                                  {"DiskResourceLockedError": ResourceIsLockedError}),
                 429: defaultdict(lambda: TooManyRequestsError),
                 500: defaultdict(lambda: InternalServerError),
                 502: defaultdict(lambda: BadGatewayError),
                 503: defaultdict(lambda: UnavailableError),
                 504: defaultdict(lambda: GatewayTimeoutError),
                 509: defaultdict(lambda: InsufficientStorageError)}

def get_exception(response):
    """
        Get an exception instance based on response, assuming the request has failed.

        :param response: an instance of :any:`requests.Response`

        :returns: an exception instance, subclass of :any:`YaDiskError`
    """

    exc_group = EXCEPTION_MAP.get(response.status_code, None)

    if exc_group is None:
        return UnknownYaDiskError("Unknown Yandex.Disk error")

    try:
        js = response.json()
    except (ValueError, RuntimeError):
        js = None

    error = ErrorObject(js)

    msg = error.message or "<empty>"
    desc = error.description or "<empty>"

    exc = exc_group[error.error]
        
    return exc(error.error, "%s (%s / %s)" % (msg, desc, error.error), response)

def auto_retry(func, n_retries=None, retry_interval=None):
    """
        Attempt to perform a request with automatic retries.
        A retry is triggered by :any:`requests.exceptions.RequestException` or :any:`RetriableYaDiskError`.

        :param func: function to run, must not require any arguments
        :param n_retries: `int`, maximum number of retries
        :param retry_interval: `int` or `float`, delay between retries (in seconds)

        :returns: return value of func()
    """

    if n_retries is None:
        n_retries = settings.DEFAULT_N_RETRIES

    if retry_interval is None:
        retry_interval = settings.DEFAULT_RETRY_INTERVAL

    for i in range(n_retries + 1):
        try:
            return func()
        except (requests.exceptions.RequestException, RetriableYaDiskError) as e:
            if i == n_retries:
                raise e

        if retry_interval:
            time.sleep(retry_interval)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..api import DiskInfoRequest

__all__ = ["get_disk_info"]

def get_disk_info(session, *args, **kwargs):
    """
        Get disk information.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param fields: list of keys to be included in the response

        :returns: :any:`DiskInfoObject`
    """

    request = DiskInfoRequest(session, *args, **kwargs)
    request.send()

    return request.process()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import requests

from ..utils import auto_retry, get_exception
from .. import settings

__all__ = ["APIRequest"]

class APIRequest(object):
    """
        Base class for all API requests.

        :param session: an instance of :any:`requests.Session`
        :param args: `dict` of arguments, that will be passed to `process_args`
        :param timeout: request timeout
        :param n_retries: maximum number of retries
        :param retry_interval: delay between retries in seconds
        :param kwargs: other parameters for session.send()

        :ivar url: `str`, request URL
        :ivar method: `str`, request method
        :ivar content_type: `str`, Content-Type header ("application/x-www-form-urlencoded" by default)
        :ivar timeout: `float` or `tuple`, request timeout
        :ivar n_retries: `int`, maximum number of retries
        :ivar success_codes: `list`-like, list of response codes that indicate request's success
        :ivar retry_interval: `float`, delay between retries in seconds
    """

    url = None
    method = None
    content_type = "application/x-www-form-urlencoded"
    timeout = None 
    n_retries = None
    success_codes = {200}
    retry_interval = None

    def __init__(self, session, args, **kwargs):
        kwargs = dict(kwargs)

        timeout = kwargs.get("timeout")
        n_retries = kwargs.pop("n_retries", None)
        retry_interval = kwargs.pop("retry_interval", None)

        if timeout is None:
            timeout = self.timeout
        if timeout is None:
            timeout = settings.DEFAULT_TIMEOUT

        kwargs["timeout"] = timeout

        if n_retries is None:
            n_retries = self.n_retries
        if n_retries is None:
            n_retries = settings.DEFAULT_N_RETRIES

        if retry_interval is None:
            retry_interval = self.retry_interval
        if retry_interval is None:
            retry_interval = settings.DEFAULT_RETRY_INTERVAL

        self.session = session
        self.args = args
        self.send_kwargs = kwargs
        self.timeout = timeout
        self.n_retries = n_retries
        self.retry_interval = retry_interval
        self.request = None
        self.response = None
        self.data = {}
        self.params = {}

        self.process_args(**self.args)
        self.prepare()

    def process_args(self):
        raise NotImplementedError

    def prepare(self):
        """Prepare the request"""

        r = requests.Request(self.method, self.url,
                             data=self.data, params=self.params)
        r.headers["Content-Type"] = self.content_type
        self.request = self.session.prepare_request(r)

    def _attempt(self):
        self.response = self.session.send(self.request, **self.send_kwargs)

        success = self.response.status_code in self.success_codes

        if not success:
            raise get_exception(self.response)

    def send(self):
        """
            Actually send the request
           
           :returns: :any:`requests.Response` (`self.response`)
        """

        auto_retry(self._attempt, self.n_retries, self.retry_interval)

        return self.response

    def process_json(self, js):
        """
            Process the JSON response.

            :param js: `dict`, JSON response

            :returns: processed response, can be anything
        """

        raise NotImplementedError

    def process(self):
        """
            Process the response.

            :returns: depends on `self.process_json()`
        """

        try:
            result = self.response.json()
        except (ValueError, RuntimeError):
            result = None
        
        if result is not None:
            return self.process_json(result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__all__ = ["YaDiskError", "RetriableYaDiskError", "UnknownYaDiskError",
           "WrongResourceTypeError",  "BadRequestError", "UnauthorizedError",
           "ForbiddenError", "NotFoundError", "NotAcceptableError",
           "ConflictError", "UnsupportedMediaError", "LockedError",
           "TooManyRequestsError", "InternalServerError", "BadGatewayError",
           "UnavailableError", "GatewayTimeoutError", "InsufficientStorageError",
           "PathNotFoundError", "ParentNotFoundError", "PathExistsError",
           "DirectoryExistsError", "FieldValidationError", "ResourceIsLockedError"]

class YaDiskError(Exception):
    """
        Base class for all exceptions in this library.

        :ivar error_type: `str`, unique error code as returned by API
        :ivar response: an instance of :any:`requests.Response`

        :param error_type: `str`, unique error code as returned by API
        :param msg: `str`, exception message
        :param response: an instance of :any:`requests.Response`
    """

    def __init__(self, error_type=None, msg="", response=None):
        Exception.__init__(self, msg)

        self.error_type = error_type
        self.response = response

class WrongResourceTypeError(YaDiskError):
    """Thrown when the resource was expected to be of different type (e.g., file instead of directory)."""

    def __init__(self, msg=""):
        YaDiskError.__init__(self, None, msg, None)

class RetriableYaDiskError(YaDiskError):
    """Thrown when there was an error but it would make sense to retry the request."""
    pass

class UnknownYaDiskError(RetriableYaDiskError):
    """Thrown when the request failed but the response does not contain any error info."""

    def __init__(self, msg="", response=None):
        RetriableYaDiskError.__init__(self, None, msg, response)

class BadRequestError(YaDiskError):
    """Thrown when the server returns code 400."""
    pass

class UnauthorizedError(YaDiskError):
    """Thrown when the server returns code 401."""
    pass

class ForbiddenError(YaDiskError):
    """Thrown when the server returns code 403."""
    pass

class NotFoundError(YaDiskError):
    """Thrown when the server returns code 404."""
    pass

class NotAcceptableError(YaDiskError):
    """Thrown when the server returns code 406."""
    pass

class ConflictError(YaDiskError):
    """Thrown when the server returns code 409."""
    pass

class UnsupportedMediaError(YaDiskError):
    """Thrown when the server returns code 415."""
    pass

class LockedError(YaDiskError):
    """Thrown when the server returns code 423."""
    pass

class TooManyRequestsError(YaDiskError):
    """Thrown when the server returns code 429."""
    pass

class InternalServerError(RetriableYaDiskError):
    """Thrown when the server returns code 500."""
    pass

class BadGatewayError(RetriableYaDiskError):
    """Thrown when the server returns code 502"""
    pass

class UnavailableError(RetriableYaDiskError):
    """Thrown when the server returns code 503."""
    pass

class GatewayTimeoutError(RetriableYaDiskError):
    """Thrown when the server returns code 504"""
    pass

class InsufficientStorageError(YaDiskError):
    """Thrown when the server returns code 509."""
    pass

class PathNotFoundError(NotFoundError):
    """Thrown when the requested path does not exist."""
    pass

class ParentNotFoundError(ConflictError):
    """Thrown by `mkdir`, `upload`, etc. when the parent directory doesn't exist."""
    pass

class PathExistsError(ConflictError):
    """Thrown when the requested path already exists."""
    pass

class DirectoryExistsError(PathExistsError):
    """Thrown when the directory already exists."""
    pass

class FieldValidationError(BadRequestError):
    """Thrown when the request contains fields with invalid data."""
    pass

class ResourceIsLockedError(LockedError):
    """Thrown when the resource is locked by another operation."""
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..api import GetOperationStatusRequest

__all__ = ["get_operation_status"]

def get_operation_status(session, operation_id, *args, **kwargs):
    """
        Get operation status.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param operation_id: ID of the operation or a link
        :param fields: list of keys to be included in the response

        :returns: `str`
    """

    request = GetOperationStatusRequest(session, operation_id, *args, **kwargs)
    request.send()

    return request.process().status



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .YaDiskObject import YaDiskObject

__all__ = ["TokenObject", "TokenRevokeStatusObject"]

class TokenObject(YaDiskObject):
    """
        Token object.

        :param token: `dict` or `None`

        :ivar access_token: `str`, token string
        :ivar refresh_token: `str`, the refresh-token
        :ivar token_type: `str`, type of the token
        :ivar expires_in: `int`, amount of time before the token expires
    """

    def __init__(self, token=None):
        YaDiskObject.__init__(self, {"access_token":  str,
                                     "refresh_token": str,
                                     "token_type":    str,
                                     "expires_in":    int})

        self.import_fields(token)

class TokenRevokeStatusObject(YaDiskObject):
    """
        Result of token revocation request.

        :param token_revoke_status: `dict` or `None`

        :ivar status: `str`, status of the operation
    """

    def __init__(self, token_revoke_status=None):
        YaDiskObject.__init__(self, {"status": str})

        self.import_fields(token_revoke_status)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pylint: disable=wildcard-import
from .YaDiskObject import *
from .ErrorObject import *
from .disk import *
from .resources import *
from .operations import *
from .auth import *



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .APIRequest import APIRequest
from ..objects import TokenObject, TokenRevokeStatusObject

__all__ = ["RefreshTokenRequest", "RevokeTokenRequest", "GetTokenRequest"]

class RefreshTokenRequest(APIRequest):
    """
        A request to refresh an existing token.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param refresh_token: the refresh token that was received with the original token
        :param client_id: application ID
        :param client_secret: application secret password

        :returns: :any:`TokenObject`
    """

    url = "https://oauth.yandex.ru/token"
    method = "POST"

    def __init__(self, session, refresh_token, client_id, client_secret, *args, **kwargs):
        APIRequest.__init__(self, session, {"refresh_token": refresh_token,
                                            "client_id":     client_id,
                                            "client_secret": client_secret}, *args, **kwargs)

    def process_args(self, refresh_token, client_id, client_secret):
        self.data["grant_type"] = "refresh_token"
        self.data["refresh_token"] = refresh_token
        self.data["client_id"] = client_id
        self.data["client_secret"] = client_secret

    def process_json(self, js):
        return TokenObject(js)

class RevokeTokenRequest(APIRequest):
    """
        A request to revoke the token.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param token: the token to be revoked
        :param client_id: application ID
        :param client_secret: application secret password

        :returns: :any:`TokenRevokeStatusObject`
    """

    url = "https://oauth.yandex.ru/revoke_token"
    method = "POST"

    def __init__(self, session, token, client_id, client_secret, *args, **kwargs):
        APIRequest.__init__(self, session, {"token":         token,
                                            "client_id":     client_id,
                                            "client_secret": client_secret}, *args, **kwargs)

    def process_args(self, token, client_id, client_secret):
        self.data["access_token"] = token
        self.data["client_id"] = client_id
        self.data["client_secret"] = client_secret

    def process_json(self, js):
        return TokenRevokeStatusObject(js)

class GetTokenRequest(APIRequest):
    """
        A request to get the token.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param code: confirmation code
        :param client_id: application ID
        :param client_secret: application secret password
        :param device_id: unique device ID (between 6 and 50 characters)

        :returns: :any:`TokenObject`
    """

    url = "https://oauth.yandex.ru/token"
    method = "POST"

    def __init__(self, session, code, client_id, client_secret,
                 device_id=None, device_name=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"code":          code,
                                            "client_id":     client_id,
                                            "client_secret": client_secret,
                                            "device_id":     device_id,
                                            "device_name":   device_name}, *args, **kwargs)

    def process_args(self, code, client_id, client_secret, device_id, device_name):
        self.data["grant_type"] = "authorization_code"
        self.data["code"] = code
        self.data["client_id"] = client_id
        self.data["client_secret"] = client_secret

        if device_id is not None:
            self.data["device_id"] = device_id

        if device_name is not None:
            self.data["device_name"] = device_name

    def process_json(self, js):
        return TokenObject(js)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

import requests

from .disk import get_disk_info
from ..api import GetTokenRequest, RefreshTokenRequest, RevokeTokenRequest
from ..exceptions import UnauthorizedError

__all__ = ["check_token", "get_auth_url", "get_code_url", "get_token",
           "refresh_token", "revoke_token"]

def check_token(session, *args, **kwargs):
    """
        Check whether the token is valid.

        :param session: an instance of :any:`requests.Session` with prepared headers

        :returns: `bool`
    """

    try:
        get_disk_info(session, *args, **kwargs)
        return True
    except UnauthorizedError:
        return False

def get_auth_url(client_id, type="code", device_id=None, device_name=None, display="popup",
                 login_hint=None, scope=None, optional_scope=None, force_confirm=True,
                 state=None):
    """
        Get authentication URL for the user to go to.

        :param client_id: application ID
        :param type: response type ("code" to get the confirmation code or "token" to get the token automatically)
        :param device_id: unique device ID, must be between 6 and 50 characters
        :param device_name: device name, should not be longer than 100 characters
        :param display: indicates whether to use lightweight layout, values other than "popup" are ignored
        :param login_hint: username or email for the account the token is being requested for
        :param scope: list of permissions for the application
        :param optional_scope: list of optional permissions for the application
        :param force_confirm: if True, user will be required to confirm access to the account
                              even if the user has already granted access for the application
        :param state: The state string, which Yandex.OAuth returns without any changes (<= 1024 characters)

        :returns: authentication URL
    """

    if type not in {"code", "token"}:
        raise ValueError("type must be either 'code' or 'token'")

    params = {"response_type": type,
              "client_id":     client_id,
              "display":       display,
              "force_confirm": "yes" if force_confirm else "no"}

    if device_id is not None:
        params["device_id"] = device_id

    if device_name is not None:
        params["device_name"] = device_name

    if login_hint is not None:
        params["login_hint"] = login_hint

    if scope is not None:
        params["scope"] = " ".join(scope)

    if optional_scope is not None:
        params["optional_scope"] = " ".join(optional_scope)

    if state is not None:
        params["state"] = state

    return "https://oauth.yandex.ru/authorize?" + urlencode(params)

def get_code_url(client_id, *args, **kwargs):
    """
        Get the URL for the user to get the confirmation code.
        The confirmation code can later be used to get the token.

        :param client_id: application ID
        :param device_id: unique device ID, must be between 6 and 50 characters
        :param device_name: device name, should not be longer than 100 characters
        :param display: indicates whether to use lightweight layout, values other than "popup" are ignored
        :param login_hint: username or email for the account the token is being requested for
        :param scope: list of permissions for the application
        :param optional_scope: list of optional permissions for the application
        :param force_confirm: if True, user will be required to confirm access to the account
                              even if the user has already granted access for the application
        :param state: The state string, which Yandex.OAuth returns without any changes (<= 1024 characters)

        :returns: authentication URL
    """

    return get_auth_url(client_id, type="code", *args, **kwargs)

def get_token(code, client_id, client_secret, *args, **kwargs):
    """
        Get a new token.

        :param code: confirmation code
        :param client_id: application ID
        :param client_secret: application secret password
        :param device_id: unique device ID (between 6 and 50 characters)

        :returns: :any:`TokenObject`
    """

    with requests.Session() as session:
        request = GetTokenRequest(session, code, client_id, client_secret, *args, **kwargs)
        request.send()

        return request.process()

def refresh_token(refresh_token, client_id, client_secret, *args, **kwargs):
    """
        Refresh an existing token.

        :param refresh_token: the refresh token that was received with the token
        :param client_id: application ID
        :param client_secret: application secret password

        :returns: :any:`TokenObject`
    """

    with requests.Session() as session:
        request = RefreshTokenRequest(session, refresh_token, client_id, client_secret, *args, **kwargs)
        request.send()

        return request.process()

def revoke_token(token, client_id, client_secret, *args, **kwargs):
    """
        Revoke the token.

        :param token: token to revoke
        :param client_id: application ID
        :param client_secret: application secret password

        :returns: :any:`TokenRevokeStatusObject`
    """

    with requests.Session() as session:
        request = RevokeTokenRequest(session, token, client_id, client_secret, *args, **kwargs)
        request.send()

        return request.process()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import collections
import json

from .APIRequest import APIRequest
from ..objects import LinkObject, PublicResourcesListObject, TrashResourceObject
from ..objects import FilesResourceListObject, LastUploadedResourceListObject
from ..objects import ResourceObject, ResourceUploadLinkObject, PublicResourceObject
from ..objects import OperationLinkObject
from ..common import is_operation_link

__all__ = ["GetPublicResourcesRequest", "UnpublishRequest", "GetDownloadLinkRequest",
           "GetTrashRequest", "RestoreTrashRequest", "DeleteTrashRequest",
           "LastUploadedRequest", "CopyRequest", "GetMetaRequest", "GetUploadLinkRequest",
           "MkdirRequest", "PublishRequest", "UploadURLRequest", "DeleteRequest",
           "SaveToDiskRequest", "GetPublicMetaRequest", "GetPublicDownloadLinkRequest",
           "MoveRequest", "FilesRequest", "PatchRequest"]

class GetPublicResourcesRequest(APIRequest):
    """
        A request to get a list of public resources.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param offset: offset from the beginning of the list
        :param limit: maximum number of elements in the list
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param type: filter based on type of resources ("file" or "dir")
        :param fields: list of keys to be included in the response

        :returns: :any:`PublicResourcesListObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources/public"
    method = "GET"

    def __init__(self, session, offset=0, limit=20, preview_size=None,
                 preview_crop=None, type=None, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"offset":       offset,
                                            "limit":        limit,
                                            "preview_size": preview_size,
                                            "preview_crop": preview_crop,
                                            "type":         type,
                                            "fields":       fields}, *args, **kwargs)

    def process_args(self, offset, limit, preview_size, preview_crop, type, fields):
        self.params["offset"] = offset
        self.params["limit"] = limit

        if preview_size is not None:
            self.params["preview_size"] = preview_size

        if preview_crop is not None:
            self.params["preview_crop"] = preview_crop

        if type is not None:
            self.params["type"] = type

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return PublicResourcesListObject(js)

class UnpublishRequest(APIRequest):
    """
        A request to make a public resource private.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource to be unpublished
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources/unpublish"
    method = "PUT"

    def __init__(self, session, path, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"path":   path,
                                            "fields": fields}, *args, **kwargs)

    def process_args(self, path, fields):
        self.params["path"] = path

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return LinkObject(js)

class GetDownloadLinkRequest(APIRequest):
    """
        A request to get a download link to a resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource to be downloaded
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources/download"
    method = "GET"

    def __init__(self, session, path, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"path": path, "fields": fields},
                            *args, **kwargs)

    def process_args(self, path, fields):
        self.params["path"] = path

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return LinkObject(js)

class GetTrashRequest(APIRequest):
    """
        A request to get meta-information about a trash resource.

        :param path: path to the trash resource
        :param limit: number of children resources to be included in the response
        :param offset: number of children resources to be skipped in the response
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param fields: list of keys to be included in the response

        :returns: :any:`TrashResourceObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/trash/resources"
    method = "GET"

    def __init__(self, session, path=None, offset=0, limit=20, sort=None,
                 preview_size=None, preview_crop=None, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"path":         path,
                                            "offset":       offset,
                                            "limit":        limit,
                                            "sort":         sort,
                                            "preview_size": preview_size,
                                            "preview_crop": preview_crop,
                                            "fields":       fields}, *args, **kwargs)

    def process_args(self, path, offset, limit, sort, preview_size, preview_crop, fields):
        self.params["path"] = path
        self.params["offset"] = offset
        self.params["limit"] = limit

        if sort is not None:
            self.params["sort"] = sort

        if preview_size is not None:
            self.params["preview_size"] = preview_size

        if preview_crop is not None:
            self.params["preview_crop"] = preview_crop

        if fields is not None:
            sub_map = {"embedded": "_embedded"}

            self.params["fields"] = ",".join(sub_map.get(f, f) for f in fields)

    def process_json(self, js):
        return TrashResourceObject(js)

class RestoreTrashRequest(APIRequest):
    """
        A request to restore trash.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the trash resource to be restored
        :param dst_path: destination path
        :param force_async: forces the operation to be executed asynchronously
        :param overwrite: `bool`, determines whether the destination can be overwritten
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject` or :any:`OperationLinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/trash/resources/restore"
    method = "PUT"
    success_codes = {201, 202}

    def __init__(self, session, path, dst_path=None, force_async=False,
                 overwrite=False, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"path":        path,
                                            "dst_path":    dst_path,
                                            "overwrite":   overwrite,
                                            "force_async": force_async,
                                            "fields":      fields}, *args, **kwargs)

    def process_args(self, path, dst_path, force_async, overwrite, fields):
        self.params["path"] = path
        self.params["overwrite"] = "true" if overwrite else "false"
        self.params["force_async"] = "true" if force_async else "false"

        if dst_path is not None:
            self.params["name"] = dst_path

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        if is_operation_link(js.get("href", "")):
            return OperationLinkObject(js)

        return LinkObject(js)

class DeleteTrashRequest(APIRequest):
    """
        A request to delete a trash resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the trash resource to be deleted
        :param force_async: forces the operation to be executed asynchronously
        :param fields: list of keys to be included in the response

        :returns: :any:`OperationLinkObject` or `None`
    """

    url = "https://cloud-api.yandex.net/v1/disk/trash/resources"
    method = "DELETE"
    success_codes = {202, 204}

    def __init__(self, session, path=None, force_async=False, fields=None,
                 *args, **kwargs):
        APIRequest.__init__(self, session, {"path":        path,
                                            "force_async": force_async,
                                            "fields":      fields}, *args, **kwargs)

    def process_args(self, path, force_async, fields):
        if path is not None:
            self.params["path"] = path

        self.params["force_async"] = "true" if force_async else "false"

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return OperationLinkObject(js)

class LastUploadedRequest(APIRequest):
    """
        A request to get the list of latest uploaded files sorted by upload date.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param limit: maximum number of elements in the list
        :param media_type: type of files to include in the list
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param fields: list of keys to be included in the response

        :returns: :any:`LastUploadedResourceListObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources/last-uploaded"
    method = "GET"

    def __init__(self, session, limit=20, media_type=None, preview_size=None,
                 preview_crop=None, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"limit":        limit,
                                            "media_type":   media_type,
                                            "preview_size": preview_size,
                                            "preview_crop": preview_crop,
                                            "fields":       fields}, *args, **kwargs)

    def process_args(self, limit, media_type, preview_size, preview_crop, fields):
        self.params["limit"] = limit

        if media_type is not None:
            if not isinstance(media_type, collections.Iterable):
                raise TypeError("media_type should be a string or an iterable")

            if isinstance(media_type, str):
                self.params["media_type"] = media_type
            else:
                self.params["media_type"] = ",".join(media_type)

        if preview_size is not None:
            self.params["preview_size"] = preview_size

        if preview_crop is not None:
            self.params["preview_crop"] = preview_crop

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return LastUploadedResourceListObject(js)

class CopyRequest(APIRequest):
    """
        A request to copy a file or a directory.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param src_path: source path
        :param dst_path: destination path
        :param overwrite: if `True` the destination path can be overwritten,
                          otherwise, an error will be raised
        :param force_async: forces the operation to be executed asynchronously
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject` or :any:`OperationLinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources/copy"
    method = "POST"
    success_codes = {201, 202}

    def __init__(self, session, src_path, dst_path, overwrite=False,
                 force_async=False, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"src_path":    src_path,
                                            "dst_path":    dst_path,
                                            "overwrite":   overwrite,
                                            "force_async": force_async,
                                            "fields":      fields}, *args, **kwargs)

    def process_args(self, src_path, dst_path, overwrite, force_async, fields):
        self.params["from"] = src_path
        self.params["path"] = dst_path
        self.params["overwrite"] = "true" if overwrite else "false"
        self.params["force_async"] = "true" if force_async else "false"

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        if is_operation_link(js["href"]):
            return OperationLinkObject(js)

        return LinkObject(js)

class GetMetaRequest(APIRequest):
    """
        A request to get meta-information about a resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource
        :param limit: number of children resources to be included in the response
        :param offset: number of children resources to be skipped in the response
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param fields: list of keys to be included in the response

        :returns: :any:`ResourceObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources"
    method = "GET"

    def __init__(self, session, path, limit=None, offset=None,
                 preview_size=None, preview_crop=None, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session,
                            {"path":         path,
                             "limit":        limit,
                             "offset":       offset,
                             "preview_size": preview_size,
                             "preview_crop": preview_crop,
                             "fields":       fields}, *args, **kwargs)

    def process_args(self, path, limit, offset, preview_size, preview_crop, fields):
        self.params["path"] = path

        if limit is not None:
            self.params["limit"] = limit

        if offset is not None:
            self.params["offset"] = offset

        if preview_size is not None:
            self.params["preview_size"] = preview_size

        if preview_crop is not None:
            self.params["preview_crop"] = "true" if preview_crop else "false"

        if fields is not None:
            sub_map = {"embedded": "_embedded"}

            self.params["fields"] = ",".join(sub_map.get(f, f) for f in fields)

    def process_json(self, js):
        return ResourceObject(js)

class GetUploadLinkRequest(APIRequest):
    """
        A request to get an upload link.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to be uploaded at
        :param overwrite: `bool`, determines whether to overwrite the destination
        :param fields: list of keys to be included in the response

        :returns: :any:`ResourceUploadLinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources/upload"
    method = "GET"

    def __init__(self, session, path, overwrite=False, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"path":      path,
                                            "overwrite": overwrite,
                                            "fields":    fields}, *args, **kwargs)

    def process_args(self, path, overwrite, fields):
        self.params["path"] = path
        self.params["overwrite"] = "true" if overwrite else "false"

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return ResourceUploadLinkObject(js)

class MkdirRequest(APIRequest):
    """
        A request to create a new directory.

        :param path: path to the directory to be created
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources"
    method = "PUT"
    success_codes = {201}

    def __init__(self, session, path, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"path": path, "fields": fields},
                            *args, **kwargs)

    def process_args(self, path, fields):
        self.params["path"] = path

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return LinkObject(js)

class PublishRequest(APIRequest):
    """
        A request to make a resource public.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource to be published
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources/publish"
    method = "PUT"

    def __init__(self, session, path, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"path":   path,
                                            "fields": fields}, *args, **kwargs)

    def process_args(self, path, fields):
        self.params["path"] = path

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return LinkObject(js)

class UploadURLRequest(APIRequest):
    """
        A request to upload a file from URL.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param url: source URL
        :param path: destination path
        :param disable_redirects: `bool`, forbid redirects
        :param fields: list of keys to be included in the response

        :returns: :any:`OperationLinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources/upload"
    method = "POST"
    success_codes = {202}

    def __init__(self, session, url, path, disable_redirects=False, fields=None,
                 *args, **kwargs):
        APIRequest.__init__(self, session, {"url":               url,
                                            "path":              path,
                                            "disable_redirects": disable_redirects,
                                            "fields":            fields}, *args, **kwargs)

    def process_args(self, url, path, disable_redirects, fields):
        self.params["url"] = url
        self.params["path"] = path
        self.params["disable_redirects"] = "true" if disable_redirects else "false"

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return OperationLinkObject(js)

class DeleteRequest(APIRequest):
    """
        A request to delete a file or a directory.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource to be removed
        :param permanently: if `True`, the resource will be removed permanently,
                            otherwise, it will be just moved to the trash
        :param force_async: forces the operation to be executed asynchronously
        :param fields: list of keys to be included in the response

        :returns: :any:`OperationLinkObject` or `None`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources"
    method = "DELETE"
    success_codes = {202, 204}

    def __init__(self, session, path, permanently=False, force_async=False,
                 fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"path":        path,
                                            "permanently": permanently,
                                            "force_async": force_async,
                                            "fields":      fields}, *args, **kwargs)

    def process_args(self, path, permanently, force_async, fields):
        self.params["path"] = path
        self.params["permanently"] = "true" if permanently else "false"
        self.params["force_async"] = "true" if force_async else "false"

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return OperationLinkObject(js)

class SaveToDiskRequest(APIRequest):
    """
        A request to save a public resource to the disk.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource
        :param name: filename of the saved resource
        :param save_path: path to the destination directory (downloads directory by default)
        :param force_async: forces the operation to be executed asynchronously
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject` or :any:`OperationLinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/public/resources/save-to-disk"
    method = "POST"
    success_codes = {201, 202}

    def __init__(self, session, public_key, name=None, path=None, save_path=None,
                 force_async=False, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"public_key":  public_key,
                                            "name":        name,
                                            "path":        path,
                                            "save_path":   save_path,
                                            "force_async": force_async,
                                            "fields":      fields}, *args, **kwargs)

    def process_args(self, public_key, name, path, save_path, force_async, fields):
        self.params["public_key"] = public_key
        
        if name is not None:
            self.params["name"] = name

        if path is not None:
            self.params["path"] = path

        if save_path is not None:
            self.params["save_path"] = save_path

        self.params["force_async"] = "true" if force_async else "false"

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        if is_operation_link(js.get("href", "")):
            return OperationLinkObject(js)

        return LinkObject(js)

class GetPublicMetaRequest(APIRequest):
    """
        A request to get meta-information about a public resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource
        :param offset: offset from the beginning of the list of nested resources
        :param limit: maximum number of nested elements to be included in the list
        :param sort: key to sort by
        :param preview_size: file preview size
        :param preview_crop: `bool`, allow preview crop
        :param fields: list of keys to be included in the response

        :returns: :any:`PublicResourceObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/public/resources"
    method = "GET"

    def __init__(self, session, public_key, offset=0, limit=20, path=None,
                 sort=None, preview_size=None, preview_crop=None, fields=None,
                 *args, **kwargs):
        APIRequest.__init__(self, session, {"public_key":   public_key,
                                            "offset":       offset,
                                            "limit":        limit,
                                            "path":         path,
                                            "sort":         sort,
                                            "preview_size": preview_size,
                                            "preview_crop": preview_crop,
                                            "fields":       fields}, *args, **kwargs)

    def process_args(self, public_key, offset, limit, path,
                     sort, preview_size, preview_crop, fields):
        self.params["public_key"] = public_key
        self.params["offset"] = offset
        self.params["limit"] = limit
        
        if path is not None:
            self.params["path"] = path

        if sort is not None:
            self.params["sort"] = sort

        if preview_size is not None:
            self.params["preview_size"] = preview_size

        if preview_crop is not None:
            self.params["preview_crop"] = preview_crop

        if fields is not None:
            sub_map = {"embedded": "_embedded",
                       "view_count": "views_count"}

            self.params["fields"] = ",".join(sub_map.get(f, f) for f in fields)

    def process_json(self, js):
        return PublicResourceObject(js)

class GetPublicDownloadLinkRequest(APIRequest):
    """
        A request to get a download link for a public resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/public/resources/download"
    method = "GET"

    def __init__(self, session, public_key, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"public_key": public_key,
                                            "fields":     fields}, *args, **kwargs)

    def process_args(self, public_key, fields):
        self.params["public_key"] = public_key
        
        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return LinkObject(js)

class MoveRequest(APIRequest):
    """
        A request to move a resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param src_path: source path to be moved
        :param dst_path: destination path
        :param force_async: forces the operation to be executed asynchronously
        :param overwrite: `bool`, determines whether to overwrite the destination
        :param fields: list of keys to be included in the response

        :returns: :any:`OperationLinkObject` or :any:`LinkObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources/move"
    method = "POST"
    success_codes = {201, 202}

    def __init__(self, session, src_path, dst_path, force_async=False,
                 overwrite=False, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"src_path":    src_path,
                                            "dst_path":    dst_path,
                                            "force_async": force_async,
                                            "overwrite":   overwrite,
                                            "fields":      fields}, *args, **kwargs)

    def process_args(self, src_path, dst_path, force_async, overwrite, fields):
        self.params["from"] = src_path
        self.params["path"] = dst_path
        self.params["overwrite"] = "true" if overwrite else "false"
        self.params["force_async"] = "true" if force_async else "false"

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        if is_operation_link(js.get("href", "")):
            return OperationLinkObject(js)

        return LinkObject(js)

class FilesRequest(APIRequest):
    """
        A request to get a flat list of all files (that doesn't include directories).

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param offset: offset from the beginning of the list
        :param limit: number of list elements to be included
        :param media_type: type of files to include in the list
        :param sort: sort type

        :returns: :any:`FilesResourceListObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources/files"
    method = "GET"

    def __init__(self, session, offset=0, limit=20, media_type=None,
                 preview_size=None, preview_crop=None, sort=None, fields=None,
                 *args, **kwargs):
        APIRequest.__init__(self, session, {"offset":       offset,
                                            "limit":        limit,
                                            "media_type":   media_type,
                                            "sort":         sort,
                                            "preview_size": preview_size,
                                            "preview_crop": preview_crop,
                                            "fields":       fields}, *args, **kwargs)

    def process_args(self, offset, limit, media_type, sort, preview_size, preview_crop, fields):
        self.params["offset"] = offset
        self.params["limit"] = limit

        if media_type is not None:
            if not isinstance(media_type, collections.Iterable):
                raise TypeError("media_type should be a string or an iterable")

            if isinstance(media_type, str):
                self.params["media_type"] = media_type
            else:
                self.params["media_type"] = ",".join(media_type)

        if preview_size is not None:
            self.params["preview_size"] = preview_size

        if preview_crop is not None:
            self.params["preview_crop"] = preview_crop

        if sort is not None:
            self.params["sort"] = sort

        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        return FilesResourceListObject(js)

class PatchRequest(APIRequest):
    """
        A request to update custom properties of a resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource
        :param properties: `dict`, custom properties to update
        :param fields: list of keys to be included in the response

        :returns: :any:`ResourceObject`
    """

    url = "https://cloud-api.yandex.net/v1/disk/resources"
    method = "PATCH"
    content_type = "application/json"

    def __init__(self, session, path, properties, fields=None, *args, **kwargs):
        APIRequest.__init__(self, session, {"path":       path,
                                            "properties": properties,
                                            "fields":     fields}, *args, **kwargs)
    def prepare(self, *args, **kwargs):
        APIRequest.prepare(self, *args, **kwargs)

        self.request.body = self.data["body"]
        self.request.headers["Content-Length"] = len(self.request.body)

    def process_args(self, path, properties, fields):
        self.params["path"] = path
        self.data["body"] = json.dumps({"custom_properties": properties}).encode("utf8")

        if fields is not None:
            sub_map = {"embedded": "_embedded"}

            self.params["fields"] = ",".join(sub_map.get(f, f) for f in fields)

    def process_json(self, js):
        return ResourceObject(js)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .YaDiskObject import YaDiskObject

__all__ = ["ErrorObject"]

class ErrorObject(YaDiskObject):
    """
        Mirrors Yandex.Disk REST API error object.

        :param error: `dict` or `None`

        :ivar message: `str`, human-readable error message
        :ivar description: `str`, technical error description
        :ivar error: `str`, error code
    """

    def __init__(self, error=None):
        YaDiskObject.__init__(self, {"message":     str,
                                     "description": str,
                                     "error":       str})
        self.set_alias("error_description", "message")
        self.import_fields(error)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from urllib.parse import urlparse, parse_qs
except ImportError:
    from urlparse import urlparse, parse_qs

from .APIRequest import APIRequest
from ..objects import OperationStatusObject

__all__ = ["GetOperationStatusRequest"]

class GetOperationStatusRequest(APIRequest):
    """
        A request to get operation status.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param operation_id: operation ID or link
        :param fields: list of keys to be included in the response

        :returns: :any:`OperationStatusObject`
    """

    method = "GET"

    def __init__(self, session, operation_id, fields=None, *args, **kwargs):
        if operation_id.startswith("https://"):
            parsed_url = urlparse(operation_id)
            self.url = parsed_url.scheme + "://" + parsed_url.netloc + parsed_url.path

            params = parse_qs(parsed_url.query)
            operation_id = parsed_url.path.rsplit("/", 1)[0]

            if fields is None:
                fields = params.get("fields", [None])[0]
        else:
            self.url = "https://cloud-api.yandex.net/v1/disk/operations/%s" % (operation_id,)

        APIRequest.__init__(self, session, {"fields": fields}, *args, **kwargs)

    def process_args(self, fields):
        if fields is not None:
            self.params["fields"] = ",".join(fields)

    def process_json(self, js):
        if "items" in js:
            return OperationStatusObject(js["items"][0])
        return OperationStatusObject(js)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import contextlib

from ..api import CopyRequest, GetDownloadLinkRequest, GetMetaRequest
from ..api import GetUploadLinkRequest, MkdirRequest, DeleteRequest, GetTrashRequest
from ..api import RestoreTrashRequest, MoveRequest, DeleteTrashRequest
from ..api import PublishRequest, UnpublishRequest, SaveToDiskRequest, GetPublicMetaRequest
from ..api import GetPublicResourcesRequest, PatchRequest, FilesRequest
from ..api import LastUploadedRequest, UploadURLRequest, GetPublicDownloadLinkRequest
from ..exceptions import WrongResourceTypeError, PathNotFoundError
from ..utils import auto_retry, get_exception

from .. import settings

__all__ = ["copy", "download", "exists", "get_download_link", "get_meta", "get_type",
           "get_upload_link", "is_dir", "is_file", "listdir", "mkdir", "remove",
           "upload", "get_trash_meta", "trash_exists", "restore_trash", "move",
           "remove_trash", "publish", "unpublish", "save_to_disk", "get_public_meta",
           "public_exists", "public_listdir", "get_public_type", "is_public_dir",
           "is_public_file", "trash_listdir", "get_trash_type", "is_trash_dir",
           "is_trash_file", "get_public_resources", "patch", "get_files",
           "get_last_uploaded", "upload_url", "get_public_download_link", "download_public"]

def copy(session, src_path, dst_path, *args, **kwargs):
    """
        Copy `src_path` to `dst_path`.
        If the operation is performed asynchronously, returns the link to the operation,
        otherwise, returns the link to the newly created resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param src_path: source path
        :param dst_path: destination path
        :param overwrite: if `True` the destination path can be overwritten,
                          otherwise, an error will be raised
        :param force_async: forces the operation to be executed asynchronously
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject` or :any:`OperationLinkObject`
    """

    request = CopyRequest(session, src_path, dst_path, *args, **kwargs)

    request.send()

    return request.process()

def download(session, src_path, file_or_path, *args, **kwargs):
    """
        Download the file.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param src_path: source path
        :param path_or_file: destination path or file-like object
    """

    kwargs = dict(kwargs)

    n_retries = kwargs.get("n_retries")

    if n_retries is None:
        n_retries = settings.DEFAULT_N_RETRIES

    retry_interval = kwargs.get("retry_interval")

    if retry_interval is None:
        retry_interval = settings.DEFAULT_RETRY_INTERVAL

    timeout = kwargs.get("timeout")

    if timeout is None:
        timeout = settings.DEFAULT_TIMEOUT

    kwargs["timeout"] = timeout

    file = None
    close_file = False

    try:
        if isinstance(file_or_path, (str, bytes)):
            close_file = True
            file = open(file_or_path, "wb")
        else:
            close_file = False
            file = file_or_path

        file_position = file.tell()

        def attempt():
            temp_kwargs = dict(kwargs)
            temp_kwargs["n_retries"] = 0
            temp_kwargs["retry_interval"] = 0.0
            link = get_download_link(session, src_path, *args, **temp_kwargs)

            # session.get() doesn't accept these parameters
            for k in ("n_retries", "retry_interval", "fields"):
                temp_kwargs.pop(k, None)

            temp_kwargs.setdefault("stream", True)

            # Disable keep-alive by default, since the download server is random
            try:
                temp_kwargs["headers"].setdefault("Connection", "close")
            except KeyError:
                temp_kwargs["headers"] = {"Connection": "close"}

            file.seek(file_position)

            with contextlib.closing(session.get(link, data=file, **temp_kwargs)) as response:
                for chunk in response.iter_content(chunk_size=8192):
                    if chunk:
                        file.write(chunk)

                if response.status_code != 200:
                    raise get_exception(response)

        auto_retry(attempt, n_retries, retry_interval)
    finally:
        if close_file and file is not None:
            file.close()

def _exists(get_meta_function, *args, **kwargs):
    try:
        get_meta_function(*args, limit=0, **kwargs)

        return True
    except PathNotFoundError:
        return False

def exists(session, path, *args, **kwargs):
    """
        Check whether `path` exists.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource

        :returns: `bool`
    """

    return _exists(get_meta, session, path, *args, **kwargs)

def get_download_link(session, path, *args, **kwargs):
    """
        Get a download link for a file (or a directory).

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource
        :param fields: list of keys to be included in the response

        :returns: `str`
    """

    request = GetDownloadLinkRequest(session, path, *args, **kwargs)
    request.send()

    return request.process().href

def get_meta(session, *args, **kwargs):
    """
        Get meta information about a file/directory.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource
        :param limit: number of children resources to be included in the response
        :param offset: number of children resources to be skipped in the response
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param fields: list of keys to be included in the response

        :returns: :any:`ResourceObject`
    """

    request = GetMetaRequest(session, *args, **kwargs)
    request.send()

    return request.process()

def _get_type(get_meta_function, session, *args, **kwargs):
    return get_meta_function(session, *args, limit=0, **kwargs).type

def get_type(session, path, *args, **kwargs):
    """
        Get resource type.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource

        :returns: "file" or "dir"
    """

    return _get_type(get_meta, session, path, *args, **kwargs)

def get_upload_link(session, path, *args, **kwargs):
    """
        Get a link to upload the file using the PUT request.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: destination path
        :param overwrite: `bool`, determines whether to overwrite the destination
        :param fields: list of keys to be included in the response

        :returns: `str`
    """

    request = GetUploadLinkRequest(session, path, *args, **kwargs)
    request.send()

    return request.process().href

def is_dir(session, path, *args, **kwargs):
    """
        Check whether `path` is a directory.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource

        :returns: `True` if `path` is a directory, `False` otherwise (even if it doesn't exist)
    """

    try:
        return get_type(session, path, *args, **kwargs) == "dir"
    except PathNotFoundError:
        return False

def is_file(session, path, *args, **kwargs):
    """
        Check whether `path` is a file.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource

        :returns: `True` if `path` is a file, `False` otherwise (even if it doesn't exist)
    """

    try:
        return get_type(session, path, *args, **kwargs) == "file"
    except PathNotFoundError:
        return False

def _listdir(get_meta_function, session, path, *args, **kwargs):
    kwargs = dict(kwargs)
    kwargs.setdefault("limit", 10000)

    result = get_meta_function(session, path, *args, **kwargs)

    if result.type == "file":
        raise WrongResourceTypeError("%r is a file" % (path,))

    for child in result.embedded.items:
        yield child

    limit = result.embedded.limit
    offset = result.embedded.offset
    total = result.embedded.total

    while offset + limit < total:
        offset += limit
        kwargs["offset"] = offset
        result = get_meta_function(session, path, *args, **kwargs)

        for child in result.embedded.items:
            yield child

        limit = result.embedded.limit
        total = result.embedded.total

def listdir(session, path, *args, **kwargs):
    """
        Get contents of `path`.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the directory
        :param limit: number of children resources to be included in the response
        :param offset: number of children resources to be skipped in the response
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param fields: list of keys to be included in the response

        :returns: generator of :any:`ResourceObject`
    """

    return _listdir(get_meta, session, path, *args, **kwargs)

def mkdir(session, path, *args, **kwargs):
    """
        Create a new directory.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the directory to be created
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject`
    """

    request = MkdirRequest(session, path, *args, **kwargs)

    request.send()

    return request.process()

def remove(session, path, *args, **kwargs):
    """
        Remove the resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource to be removed
        :param permanently: if `True`, the resource will be removed permanently,
                            otherwise, it will be just moved to the trash
        :param force_async: forces the operation to be executed asynchronously
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject` if the operation is performed asynchronously, `None` otherwise
    """

    request = DeleteRequest(session, path, *args, **kwargs)

    request.send()

    return request.process()

def upload(session, file_or_path, dst_path, *args, **kwargs):
    """
        Upload a file to disk.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path_or_file: path or file-like object to be uploaded
        :param dst_path: destination path
        :param overwrite: if `True`, the resource will be overwritten if it already exists,
                          an error will be raised otherwise
    """

    kwargs = dict(kwargs)

    timeout = kwargs.get("timeout")

    if timeout is None:
        timeout = settings.DEFAULT_UPLOAD_TIMEOUT

    retry_interval = kwargs.get("retry_interval")

    if retry_interval is None:
        retry_interval = settings.DEFAULT_UPLOAD_RETRY_INTERVAL

    n_retries = kwargs.get("n_retries")

    if n_retries is None:
        n_retries = settings.DEFAULT_N_RETRIES

    kwargs["timeout"] = timeout

    file = None
    close_file = False

    try:
        if isinstance(file_or_path, (str, bytes)):
            close_file = True
            file = open(file_or_path, "rb")
        else:
            close_file = False
            file = file_or_path

        file_position = file.tell()

        def attempt():
            temp_kwargs = dict(kwargs)
            temp_kwargs["n_retries"] = 0
            temp_kwargs["retry_interval"] = 0.0

            link = get_upload_link(session, dst_path, *args, **temp_kwargs)

            # session.put() doesn't accept these parameters
            for k in ("n_retries", "retry_interval", "overwrite", "fields"):
                temp_kwargs.pop(k, None)

            temp_kwargs.setdefault("stream", True)

            # Disable keep-alive by default, since the upload server is random
            try:
                temp_kwargs["headers"].setdefault("Connection", "close")
            except KeyError:
                temp_kwargs["headers"] = {"Connection": "close"}

            file.seek(file_position)

            with contextlib.closing(session.put(link, data=file, **temp_kwargs)) as response:
                if response.status_code != 201:
                    raise get_exception(response)

        auto_retry(attempt, n_retries, retry_interval)
    finally:
        if close_file and file is not None:
            file.close()

def get_trash_meta(session, path, *args, **kwargs):
    """
        Get meta information about a trash resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the trash resource
        :param limit: number of children resources to be included in the response
        :param offset: number of children resources to be skipped in the response
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param fields: list of keys to be included in the response

        :returns: :any:`TrashResourceObject`
    """

    request = GetTrashRequest(session, path, *args, **kwargs)

    request.send()

    return request.process()

def trash_exists(session, path, *args, **kwargs):
    """
        Check whether the trash resource at `path` exists.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the trash resource

        :returns: `bool`
    """

    return _exists(get_trash_meta, session, path, *args, **kwargs)

def restore_trash(session, path, *args, **kwargs):
    """
        Restore a trash resource.
        Returns a link to the newly created resource or a link to the asynchronous operation.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the trash resource to be restored
        :param dst_path: destination path
        :param overwrite: `bool`, determines whether the destination can be overwritten
        :param force_async: forces the operation to be executed asynchronously
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject` or :any:`OperationLinkObject`
    """

    request = RestoreTrashRequest(session, path, *args, **kwargs)
    request.send()

    return request.process()

def move(session, src_path, dst_path, *args, **kwargs):
    """
        Move `src_path` to `dst_path`.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param src_path: source path to be moved
        :param dst_path: destination path
        :param overwrite: `bool`, determines whether to overwrite the destination
        :param force_async: forces the operation to be executed asynchronously
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject` or :any:`OperationLinkObject`
    """

    request = MoveRequest(session, src_path, dst_path, *args, **kwargs)
    request.send()

    return request.process()

def remove_trash(session, path, *args, **kwargs):
    """
        Remove a trash resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the trash resource to be deleted
        :param force_async: forces the operation to be executed asynchronously
        :param fields: list of keys to be included in the response

        :returns: :any:`OperationLinkObject` if the operation is performed asynchronously, `None` otherwise
    """

    request = DeleteTrashRequest(session, path, *args, **kwargs)
    request.send()

    return request.process()

def publish(session, path, *args, **kwargs):
    """
        Make a resource public.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource to be published
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject`, link to the resource
    """

    request = PublishRequest(session, path, *args, **kwargs)
    request.send()

    return request.process()

def unpublish(session, path, *args, **kwargs):
    """
        Make a public resource private.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource to be unpublished
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject`
    """

    request = UnpublishRequest(session, path, *args, **kwargs)
    request.send()

    return request.process()

def save_to_disk(session, public_key, *args, **kwargs):
    """
        Saves a public resource to the disk.
        Returns the link to the operation if it's performed asynchronously,
        or a link to the resource otherwise.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource
        :param name: filename of the saved resource
        :param save_path: path to the destination directory (downloads directory by default)
        :param force_async: forces the operation to be executed asynchronously
        :param fields: list of keys to be included in the response

        :returns: :any:`LinkObject` or :any:`OperationLinkObject`
    """

    request = SaveToDiskRequest(session, public_key, *args, **kwargs)
    request.send()

    return request.process()

def get_public_meta(session, public_key, *args, **kwargs):
    """
        Get meta-information about a public resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource
        :param offset: offset from the beginning of the list of nested resources
        :param limit: maximum number of nested elements to be included in the list
        :param sort: key to sort by
        :param preview_size: file preview size
        :param preview_crop: `bool`, allow preview crop
        :param fields: list of keys to be included in the response

        :returns: :any:`PublicResourceObject`
    """

    request = GetPublicMetaRequest(session, public_key, *args, **kwargs)
    request.send()

    return request.process()

def public_exists(session, public_key, *args, **kwargs):
    """
        Check whether the public resource exists.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource

        :returns: `bool`
    """

    return _exists(get_public_meta, session, public_key, *args, **kwargs)

def public_listdir(session, public_key, *args, **kwargs):
    """
        Get contents of a public directory.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource
        :param limit: number of children resources to be included in the response
        :param offset: number of children resources to be skipped in the response
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param fields: list of keys to be included in the response

        :returns: generator of :any:`PublicResourceObject`
    """

    return _listdir(get_public_meta, session, public_key, *args, **kwargs)

def get_public_type(session, public_key, *args, **kwargs):
    """
        Get public resource type.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource

        :returns: "file" or "dir"
    """

    return _get_type(get_public_meta, session, public_key, *args, **kwargs)

def is_public_dir(session, public_key, *args, **kwargs):
    """
        Check whether `public_key` is a public directory.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource

        :returns: `True` if `public_key` is a directory, `False` otherwise (even if it doesn't exist)
    """

    try:
        return get_public_type(session, public_key, *args, **kwargs) == "dir"
    except PathNotFoundError:
        return False

def is_public_file(session, public_key, *args, **kwargs):
    """
        Check whether `public_key` is a public file.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource

        :returns: `True` if `public_key` is a file, `False` otherwise (even if it doesn't exist)
    """

    try:
        return get_public_type(session, public_key, *args, **kwargs) == "file"
    except PathNotFoundError:
        return False

def trash_listdir(session, path, *args, **kwargs):
    """
        Get contents of a trash resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the directory in the trash bin
        :param limit: number of children resources to be included in the response
        :param offset: number of children resources to be skipped in the response
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param fields: list of keys to be included in the response

        :returns: generator of :any:`TrashResourceObject`
    """

    return _listdir(get_trash_meta, session, path, *args, **kwargs)

def get_trash_type(session, path, *args, **kwargs):
    """
        Get trash resource type.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the trash resource

        :returns: "file" or "dir"
    """

    return _get_type(get_trash_meta, session, path, *args, **kwargs)

def is_trash_dir(session, path, *args, **kwargs):
    """
        Check whether `path` is a trash directory.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the trash resource

        :returns: `True` if `path` is a directory, `False` otherwise (even if it doesn't exist)
    """

    try:
        return get_trash_type(session, path, *args, **kwargs) == "dir"
    except PathNotFoundError:
        return False

def is_trash_file(session, path, *args, **kwargs):
    """
        Check whether `path` is a trash file.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the trash resource

        :returns: `True` if `path` is a file, `False` otherwise (even if it doesn't exist)
    """

    try:
        return get_trash_type(session, path, *args, **kwargs) == "file"
    except PathNotFoundError:
        return False

def get_public_resources(session, *args, **kwargs):
    """
        Get a list of public resources.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param offset: offset from the beginning of the list
        :param limit: maximum number of elements in the list
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param type: filter based on type of resources ("file" or "dir")
        :param fields: list of keys to be included in the response

        :returns: :any:`PublicResourcesListObject`
    """

    request = GetPublicResourcesRequest(session, *args, **kwargs)
    request.send()

    return request.process()

def patch(session, path, properties, *args, **kwargs):
    """
        Update custom properties of a resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param path: path to the resource
        :param properties: `dict`, custom properties to update
        :param fields: list of keys to be included in the response

        :returns: :any:`ResourceObject`
    """

    request = PatchRequest(session, path, properties, *args, **kwargs)
    request.send()

    return request.process()

def get_files(session, *args, **kwargs):
    """
        Get a flat list of all files (that doesn't include directories).

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param offset: offset from the beginning of the list
        :param limit: number of list elements to be included
        :param media_type: type of files to include in the list
        :param sort: sort type

        :returns: generator of :any:`ResourceObject`
    """

    if kwargs.get("limit") is not None:
        request = FilesRequest(session, *args, **kwargs)
        request.send()

        for i in request.process().items:
            yield i

        return

    kwargs = dict(kwargs)

    kwargs.setdefault("offset", 0)
    kwargs["limit"] = 1000

    while True:
        counter = 0
        for i in get_files(session, *args, **kwargs):
            counter += 1
            yield i

        if counter < kwargs["limit"]:
            break

        kwargs["offset"] += kwargs["limit"]

def get_last_uploaded(session, *args, **kwargs):
    """
        Get the list of latest uploaded files sorted by upload date.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param limit: maximum number of elements in the list
        :param media_type: type of files to include in the list
        :param preview_size: size of the file preview
        :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
        :param fields: list of keys to be included in the response

        :returns: generator of :any:`LastUploadedResourceListObject`
    """

    request = LastUploadedRequest(session, *args, **kwargs)
    request.send()

    for i in request.process().items:
        yield i

def upload_url(session, url, path, *args, **kwargs):
    """
        Upload a file from URL.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param url: source URL
        :param path: destination path
        :param disable_redirects: `bool`, forbid redirects
        :param fields: list of keys to be included in the response

        :returns: :any:`OperationLinkObject`, link to the asynchronous operation
    """

    request = UploadURLRequest(session, url, path, *args, **kwargs)
    request.send()

    return request.process()

def get_public_download_link(session, public_key, *args, **kwargs):
    """
        Get a download link for a public resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource
        :param fields: list of keys to be included in the response

        :returns: `str`
    """

    request = GetPublicDownloadLinkRequest(session, public_key, *args, **kwargs)
    request.send()

    return request.process().href

def download_public(session, public_key, file_or_path, *args, **kwargs):
    """
        Download the public resource.

        :param session: an instance of :any:`requests.Session` with prepared headers
        :param public_key: public key or public URL of the resource
        :param path_or_file: destination path or file-like object
    """

    n_retries = kwargs.pop("n_retries", None)

    if n_retries is None:
        n_retries = settings.DEFAULT_N_RETRIES

    retry_interval = kwargs.pop("retry_interval", None)

    if retry_interval is None:
        retry_interval = settings.DEFAULT_RETRY_INTERVAL

    file = None
    close_file = False

    try:
        if isinstance(file_or_path, (str, bytes)):
            close_file = True
            file = open(file_or_path, "wb")
        else:
            close_file = False
            file = file_or_path

        file_position = file.tell()

        def attempt():
            temp_kwargs = dict(kwargs)
            temp_kwargs["n_retries"] = 0
            temp_kwargs["retry_interval"] = 0.0

            link = get_public_download_link(session, public_key, *args, **temp_kwargs)

            temp_kwargs.pop("n_retries", None)
            temp_kwargs.pop("retry_interval", None)

            timeout = temp_kwargs.get("timeout")

            if timeout is None:
                timeout = settings.DEFAULT_TIMEOUT

            temp_kwargs["timeout"] = timeout
            temp_kwargs.setdefault("stream", True)

            # Disable keep-alive by default, since the download server is random
            try:
                temp_kwargs["headers"].setdefault("Connection", "close")
            except KeyError:
                temp_kwargs["headers"] = {"Connection": "close"}

            file.seek(file_position)

            with contextlib.closing(session.get(link, data=file, **temp_kwargs)) as response:
                for chunk in response.iter_content(chunk_size=8192):
                    if chunk:
                        file.write(chunk)

                if response.status_code != 200:
                    raise get_exception(response)

        auto_retry(attempt, n_retries, retry_interval)
    finally:
        if close_file and file is not None:
            file.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import functools
import threading

import requests

from . import functions

__all__ = ["YaDisk"]

class SelfDestructingSession(requests.Session):
    """Just like your regular :any:`requests.Session` but with a destructor"""

    def __init__(self, *args, **kwargs):
        self._init_completed = False

        requests.Session.__init__(self, *args, **kwargs)

        self._init_completed = True

    def __del__(self):
        # Extra check to avoid AttributeError
        if self._init_completed:
            if hasattr(requests.Session, "__del__"):
                requests.Session.__del__(self)

            self.close()

class YaDisk(object):
    """
        Implements access to Yandex.Disk REST API.

        :param id: application ID
        :param secret: application secret password
        :param token: application token

        :ivar id: `str`, application ID
        :ivar secret: `str`, application secret password
        :ivar token: `str`, application token
    """

    def __init__(self, id="", secret="", token=""):
        self.id = id
        self.secret = secret
        self.token = token

        @functools.lru_cache(maxsize=1024)
        def _get_session(token, tid):
            return self.make_session(token)

        self._get_session = _get_session

    def clear_session_cache(self):
        """Clears the session cache. Unused sessions will be closed."""

        self._get_session.cache_clear()

    def make_session(self, token=None):
        """
            Prepares :any:`requests.Session` object with headers needed for API.
            
            :param token: application token, equivalent to `self.token` if `None`
            :returns: :any:`requests.Session`
        """

        if token is None:
            token = self.token

        session = SelfDestructingSession()

        if token:
            session.headers["Authorization"] = "OAuth " + token

        return session

    def get_session(self, token=None):
        """
            Like :any:`YaDisk.make_session` but wrapped in :any:`functools.lru_cache`.
            
            :returns: :any:`requests.Session`, different instances for different threads
        """

        if token is None:
            token = self.token

        return self._get_session(token, threading.get_ident())

    def get_auth_url(self, *args, **kwargs):
        """
            Get authentication URL for the user to go to.

            :param type: response type ("code" to get the confirmation code or "token" to get the token automatically)
            :param device_id: unique device ID, must be between 6 and 50 characters
            :param device_name: device name, should not be longer than 100 characters
            :param display: indicates whether to use lightweight layout, values other than "popup" are ignored
            :param login_hint: username or email for the account the token is being requested for
            :param scope: list of permissions for the application
            :param optional_scope: list of optional permissions for the application
            :param force_confirm: if True, user will be required to confirm access to the account
                                  even if the user has already granted access for the application
            :param state: The state string, which Yandex.OAuth returns without any changes (<= 1024 characters)

            :returns: authentication URL
        """

        return functions.get_auth_url(self.id, *args, **kwargs)

    def get_code_url(self, *args, **kwargs):
        """
            Get the URL for the user to get the confirmation code.
            The confirmation code can later be used to get the token.

            :param device_id: unique device ID, must be between 6 and 50 characters
            :param device_name: device name, should not be longer than 100 characters
            :param display: indicates whether to use lightweight layout, values other than "popup" are ignored
            :param login_hint: username or email for the account the token is being requested for
            :param scope: list of permissions for the application
            :param optional_scope: list of optional permissions for the application
            :param force_confirm: if True, user will be required to confirm access to the account
                                  even if the user has already granted access for the application
            :param state: The state string, which Yandex.OAuth returns without any changes (<= 1024 characters)

            :returns: authentication URL
        """

        return functions.get_code_url(self.id, *args, **kwargs)

    def get_token(self, code, *args, **kwargs):
        """
            Get a new token.

            :param code: confirmation code
            :param device_id: unique device ID (between 6 and 50 characters)

            :returns: :any:`TokenObject`
        """

        return functions.get_token(code, self.id, self.secret, *args, **kwargs)

    def refresh_token(self, refresh_token, *args, **kwargs):
        """
            Refresh an existing token.

            :param refresh_token: the refresh token that was received with the token

            :returns: :any:`TokenObject`
        """

        return functions.refresh_token(refresh_token, self.id, self.secret, *args, **kwargs)

    def revoke_token(self, token=None, *args, **kwargs):
        """
            Revoke the token.

            :param token: token to revoke, equivalent to `self.token` if `None`

            :returns: :any:`TokenRevokeStatusObject`
        """

        if token is None:
            token = self.token

        return functions.revoke_token(token, self.id, self.secret, *args, **kwargs)

    def get_disk_info(self, *args, **kwargs):
        """
            Get disk information.

            :param fields: list of keys to be included in the response

            :returns: :any:`DiskInfoObject`
        """

        return functions.get_disk_info(self.get_session(), *args, **kwargs)

    def get_meta(self, path, *args, **kwargs):
        """
            Get meta information about a file/directory.

            :param path: path to the resource
            :param limit: number of children resources to be included in the response
            :param offset: number of children resources to be skipped in the response
            :param preview_size: size of the file preview
            :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
            :param fields: list of keys to be included in the response

            :returns: :any:`ResourceObject`
        """

        return functions.get_meta(self.get_session(), path, *args, **kwargs)

    def exists(self, path, *args, **kwargs):
        """
            Check whether `path` exists.

            :param path: path to the resource

            :returns: `bool`
        """

        return functions.exists(self.get_session(), path, *args, **kwargs)

    def get_type(self, path, *args, **kwargs):
        """
            Get resource type.

            :param path: path to the resource

            :returns: "file" or "dir"
        """

        return functions.get_type(self.get_session(), path, *args, **kwargs)

    def is_file(self, path, *args, **kwargs):
        """
            Check whether `path` is a file.

            :param path: path to the resource

            :returns: `True` if `path` is a file, `False` otherwise (even if it doesn't exist)
        """

        return functions.is_file(self.get_session(), path, *args, **kwargs)

    def is_dir(self, path, *args, **kwargs):
        """
            Check whether `path` is a directory.

            :param path: path to the resource

            :returns: `True` if `path` is a directory, `False` otherwise (even if it doesn't exist)
        """

        return functions.is_dir(self.get_session(), path, *args, **kwargs)

    def listdir(self, path, *args, **kwargs):
        """
            Get contents of `path`.

            :param path: path to the directory
            :param limit: number of children resources to be included in the response
            :param offset: number of children resources to be skipped in the response
            :param preview_size: size of the file preview
            :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
            :param fields: list of keys to be included in the response

            :returns: generator of :any:`ResourceObject`
        """

        return functions.listdir(self.get_session(), path, *args, **kwargs)

    def get_upload_link(self, path, *args, **kwargs):
        """
            Get a link to upload the file using the PUT request.

            :param path: destination path
            :param overwrite: `bool`, determines whether to overwrite the destination
            :param fields: list of keys to be included in the response

            :returns: `str`
        """

        return functions.get_upload_link(self.get_session(), path, *args, **kwargs)

    def upload(self, path_or_file, dst_path, *args, **kwargs):
        """
            Upload a file to disk.

            :param path_or_file: path or file-like object to be uploaded
            :param dst_path: destination path
            :param overwrite: if `True`, the resource will be overwritten if it already exists,
                              an error will be raised otherwise
            :param fields: list of keys to be included in the response
        """

        functions.upload(self.get_session(), path_or_file, dst_path, *args, **kwargs)

    def get_download_link(self, path, *args, **kwargs):
        """
            Get a download link for a file (or a directory).

            :param path: path to the resource
            :param fields: list of keys to be included in the response

            :returns: `str`
        """

        return functions.get_download_link(self.get_session(), path, *args, **kwargs)

    def download(self, src_path, path_or_file, *args, **kwargs):
        """
            Download the file.

            :param src_path: source path
            :param path_or_file: destination path or file-like object
        """

        functions.download(self.get_session(), src_path, path_or_file, *args, **kwargs)

    def remove(self, path, *args, **kwargs):
        """
            Remove the resource.

            :param path: path to the resource to be removed
            :param permanently: if `True`, the resource will be removed permanently,
                                otherwise, it will be just moved to the trash
            :param force_async: forces the operation to be executed asynchronously
            :param fields: list of keys to be included in the response

            :returns: :any:`OperationLinkObject` if the operation is performed asynchronously, `None` otherwise
        """

        return functions.remove(self.get_session(), path, *args, **kwargs)

    def mkdir(self, path, *args, **kwargs):
        """
            Create a new directory.

            :param path: path to the directory to be created
            :param fields: list of keys to be included in the response

            :returns: :any:`LinkObject`
        """

        return functions.mkdir(self.get_session(), path, *args, **kwargs)

    def check_token(self, token=None, *args, **kwargs):
        """
            Check whether the token is valid.

            :param token: token to check, equivalent to `self.token` if `None`

            :returns: `bool`
        """

        return functions.check_token(self.get_session(token), *args, **kwargs)

    def get_trash_meta(self, path, *args, **kwargs):
        """
            Get meta information about a trash resource.

            :param path: path to the trash resource
            :param limit: number of children resources to be included in the response
            :param offset: number of children resources to be skipped in the response
            :param preview_size: size of the file preview
            :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
            :param fields: list of keys to be included in the response

            :returns: :any:`TrashResourceObject`
        """

        return functions.get_trash_meta(self.get_session(), path, *args, **kwargs)

    def trash_exists(self, path, *args, **kwargs):
        """
            Check whether the trash resource at `path` exists.

            :param path: path to the trash resource

            :returns: `bool`
        """

        return functions.trash_exists(self.get_session(), path, *args, **kwargs)

    def get_operation_status(self, operation_id, *args, **kwargs):
        """
            Get operation status.

            :param operation_id: ID of the operation or a link
            :param fields: list of keys to be included in the response

            :returns: `str`
        """

        return functions.get_operation_status(self.get_session(), operation_id, *args, **kwargs)

    def copy(self, src_path, dst_path, *args, **kwargs):
        """
            Copy `src_path` to `dst_path`.
            If the operation is performed asynchronously, returns the link to the operation,
            otherwise, returns the link to the newly created resource.

            :param src_path: source path
            :param dst_path: destination path
            :param overwrite: if `True` the destination path can be overwritten,
                              otherwise, an error will be raised
            :param force_async: forces the operation to be executed asynchronously
            :param fields: list of keys to be included in the response

            :returns: :any:`LinkObject` or :any:`OperationLinkObject`
        """

        return functions.copy(self.get_session(), src_path, dst_path, *args, **kwargs)

    def restore_trash(self, path, *args, **kwargs):
        """
            Restore a trash resource.
            Returns a link to the newly created resource or a link to the asynchronous operation.

            :param path: path to the trash resource to restore
            :param dst_path: destination path
            :param overwrite: `bool`, determines whether the destination can be overwritten
            :param force_async: forces the operation to be executed asynchronously
            :param fields: list of keys to be included in the response

            :returns: :any:`LinkObject` or :any:`OperationLinkObject`
        """

        return functions.restore_trash(self.get_session(), path, *args, **kwargs)

    def move(self, src_path, dst_path, *args, **kwargs):
        """
            Move `src_path` to `dst_path`.

            :param src_path: source path to be moved
            :param dst_path: destination path
            :param overwrite: `bool`, determines whether to overwrite the destination
            :param force_async: forces the operation to be executed asynchronously
            :param fields: list of keys to be included in the response

            :returns: :any:`OperationLinkObject` or :any:`LinkObject`
        """

        return functions.move(self.get_session(), src_path, dst_path, *args, **kwargs)

    def remove_trash(self, path, *args, **kwargs):
        """
            Remove a trash resource.

            :param path: path to the trash resource to be deleted
            :param force_async: forces the operation to be executed asynchronously
            :param fields: list of keys to be included in the response

            :returns: :any:`OperationLinkObject` if the operation is performed asynchronously, `None` otherwise
        """

        return functions.remove_trash(self.get_session(), path, *args, **kwargs)

    def publish(self, path, *args, **kwargs):
        """
            Make a resource public.

            :param path: path to the resource to be published
            :param fields: list of keys to be included in the response

            :returns: :any:`LinkObject`, link to the resource
        """

        return functions.publish(self.get_session(), path, *args, **kwargs)

    def unpublish(self, path, *args, **kwargs):
        """
            Make a public resource private.

            :param path: path to the resource to be unpublished
            :param fields: list of keys to be included in the response

            :returns: :any:`LinkObject`, link to the resource
        """

        return functions.unpublish(self.get_session(), path, *args, **kwargs)

    def save_to_disk(self, public_key, *args, **kwargs):
        """
            Saves a public resource to the disk.
            Returns the link to the operation if it's performed asynchronously,
            or a link to the resource otherwise.

            :param public_key: public key or public URL of the resource
            :param name: filename of the saved resource
            :param save_path: path to the destination directory (downloads directory by default)
            :param force_async: forces the operation to be executed asynchronously
            :param fields: list of keys to be included in the response

            :returns: :any:`LinkObject` or :any:`OperationLinkObject`
        """

        return functions.save_to_disk(self.get_session(), public_key, *args, **kwargs)

    def get_public_meta(self, public_key, *args, **kwargs):
        """
            Get meta-information about a public resource.

            :param public_key: public key or public URL of the resource
            :param offset: offset from the beginning of the list of nested resources
            :param limit: maximum number of nested elements to be included in the list
            :param sort: key to sort by
            :param preview_size: file preview size
            :param preview_crop: `bool`, allow preview crop
            :param fields: list of keys to be included in the response

            :returns: :any:`PublicResourceObject`
        """

        return functions.get_public_meta(self.get_session(), public_key, *args, **kwargs)

    def public_exists(self, public_key, *args, **kwargs):
        """
            Check whether the public resource exists.

            :param public_key: public key or public URL of the resource

            :returns: `bool`
        """

        return functions.public_exists(self.get_session(), public_key, *args, **kwargs)

    def public_listdir(self, public_key, *args, **kwargs):
        """
            Get contents of a public directory.

            :param public_key: public key or public URL of the resource
            :param limit: number of children resources to be included in the response
            :param offset: number of children resources to be skipped in the response
            :param preview_size: size of the file preview
            :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
            :param fields: list of keys to be included in the response

            :returns: generator of :any:`PublicResourceObject`
        """

        return functions.public_listdir(self.get_session(), public_key, *args, **kwargs)

    def get_public_type(self, public_key, *args, **kwargs):
        """
            Get public resource type.

            :param public_key: public key or public URL of the resource

            :returns: "file" or "dir"
        """

        return functions.get_public_type(self.get_session(), public_key, *args, **kwargs)

    def is_public_dir(self, public_key, *args, **kwargs):
        """
            Check whether `public_key` is a public directory.

            :param public_key: public key or public URL of the resource

            :returns: `True` if `public_key` is a directory, `False` otherwise (even if it doesn't exist)
        """

        return functions.is_public_dir(self.get_session(), public_key, *args, **kwargs)

    def is_public_file(self, public_key, *args, **kwargs):
        """
            Check whether `public_key` is a public file.

            :param public_key: public key or public URL of the resource

            :returns: `True` if `public_key` is a file, `False` otherwise (even if it doesn't exist)
        """

        return functions.is_public_file(self.get_session(), public_key, *args, **kwargs)

    def trash_listdir(self, path, *args, **kwargs):
        """
            Get contents of a trash resource.

            :param path: path to the directory in the trash bin
            :param limit: number of children resources to be included in the response
            :param offset: number of children resources to be skipped in the response
            :param preview_size: size of the file preview
            :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
            :param fields: list of keys to be included in the response

            :returns: generator of :any:`TrashResourceObject`
        """

        return functions.trash_listdir(self.get_session(), path, *args, **kwargs)

    def get_trash_type(self, path, *args, **kwargs):
        """
            Get trash resource type.

            :param path: path to the trash resource

            :returns: "file" or "dir"
        """

        return functions.get_trash_type(self.get_session(), path, *args, **kwargs)

    def is_trash_dir(self, path, *args, **kwargs):
        """
            Check whether `path` is a trash directory.

            :param path: path to the trash resource

            :returns: `True` if `path` is a directory, `False` otherwise (even if it doesn't exist)
        """

        return functions.is_trash_dir(self.get_session(), path, *args, **kwargs)

    def is_trash_file(self, path, *args, **kwargs):
        """
            Check whether `path` is a trash file.

            :param path: path to the trash resource

            :returns: `True` if `path` is a directory, `False` otherwise (even if it doesn't exist)
        """

        return functions.is_trash_file(self.get_session(), path, *args, **kwargs)

    def get_public_resources(self, *args, **kwargs):
        """
            Get a list of public resources.

            :param offset: offset from the beginning of the list
            :param limit: maximum number of elements in the list
            :param preview_size: size of the file preview
            :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
            :param type: filter based on type of resources ("file" or "dir")
            :param fields: list of keys to be included in the response

            :returns: :any:`PublicResourcesListObject`
        """

        return functions.get_public_resources(self.get_session(), *args, **kwargs)

    def patch(self, path, properties, *args, **kwargs):
        """
            Update custom properties of a resource.

            :param path: path to the resource
            :param properties: `dict`, custom properties to update
            :param fields: list of keys to be included in the response

            :returns: :any:`ResourceObject`
        """

        return functions.patch(self.get_session(), path, properties, *args, **kwargs)

    def get_files(self, *args, **kwargs):
        """
            Get a flat list of all files (that doesn't include directories).

            :param offset: offset from the beginning of the list
            :param limit: number of list elements to be included
            :param media_type: type of files to include in the list
            :param sort: sort type

            :returns: generator of :any:`ResourceObject`
        """

        return functions.get_files(self.get_session(), *args, **kwargs)

    def get_last_uploaded(self, *args, **kwargs):
        """
            Get the list of latest uploaded files sorted by upload date.

            :param limit: maximum number of elements in the list
            :param media_type: type of files to include in the list
            :param preview_size: size of the file preview
            :param preview_crop: `bool`, cut the preview to the size specified in the `preview_size`
            :param fields: list of keys to be included in the response

            :returns: generator of :any:`LastUploadedResourceListObject`
        """

        return functions.get_last_uploaded(self.get_session(), *args, **kwargs)

    def upload_url(self, url, path, *args, **kwargs):
        """
            Upload a file from URL.

            :param url: source URL
            :param path: destination path
            :param disable_redirects: `bool`, forbid redirects
            :param fields: list of keys to be included in the response

            :returns: :any:`OperationLinkObject`, link to the asynchronous operation
        """

        return functions.upload_url(self.get_session(), url, path, *args, **kwargs)

    def get_public_download_link(self, public_key, *args, **kwargs):
        """
            Get a download link for a public resource.

            :param public_key: public key or public URL of the resource
            :param fields: list of keys to be included in the response

            :returns: `str`
        """

        return functions.get_public_download_link(self.get_session(), public_key, *args, **kwargs)

    def download_public(self, public_key, file_or_path, *args, **kwargs):
        """
            Download the public resource.

            :param public_key: public key or public URL of the resource
            :param path_or_file: destination path or file-like object
        """

        functions.download_public(self.get_session(), public_key, file_or_path, *args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .YaDiskObject import YaDiskObject
from .resources import LinkObject

__all__ = ["OperationStatusObject"]

class OperationStatusObject(YaDiskObject):
    """
        Operation status object.

        :param operation_status: `dict` or `None`

        :ivar type: `str`, type of the operation
        :ivar status: `str`, status of the operation
        :ivar operation_id: `str`, ID of the operation
        :ivar link: :any:`LinkObject`, link to the operation
        :ivar data: `dict`, other information about the operation
    """

    def __init__(self, operation_status=None):
        YaDiskObject.__init__(self, {"type":         str,
                                     "status":       str,
                                     "operation_id": str,
                                     "link":         LinkObject,
                                     "data":         dict})

        self.import_fields(operation_status)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
import os

from setuptools import setup, find_packages

module_dir = os.path.dirname(__file__)

with codecs.open(os.path.join(module_dir, "README.rst"), encoding="utf8") as f:
    long_description = f.read()

setup(name="yadisk",
      version="1.2.9",
      packages=find_packages(exclude=("tests",)),
      description="Библиотека-клиент REST API Яндекс.Диска / Yandex.Disk REST API client library",
      long_description=long_description,
      author="Ivan Konovalov",
      author_email="ivknv0@gmail.com",
      license="LGPLv3",
      python_requires=">=3",
      install_requires=["requests"],
      url="https://github.com/ivknv/yadisk",
      project_urls={"Source code": "https://github.com/ivknv/yadisk",
                    "Documentation (EN)": "https://yadisk.readthedocs.io/en/latest",
                    "Documentation (RU)": "https://yadisk.readthedocs.io/ru/latest"},
      classifiers=[
          "Development Status :: 4 - Beta",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
          "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
          "Programming Language :: Python",
          "Programming Language :: Python :: 3",
          "Operating System :: OS Independent",
          "Topic :: Internet",
          "Topic :: Software Development :: Libraries",
          "Topic :: Software Development :: Libraries :: Python Modules"],
      keywords="yandex yandex.disk rest")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .YaDiskObject import YaDiskObject

__all__ = ["DiskInfoObject", "SystemFoldersObject", "UserObject"]

class DiskInfoObject(YaDiskObject):
    """
        Disk information object.

        :param disk_info: `dict` or `None`

        :ivar max_file_size: `int`, maximum supported file size (bytes)
        :ivar total_space: `int`, total disk size (bytes)
        :ivar trash_size: `int`, amount of space used by trash (bytes), part of `used_space`
        :ivar is_paid: `bool`, tells if the account is paid or not
        :ivar used_space: `int`, amount of space used (bytes)
        :ivar system_folders: :any:`SystemFoldersObject`, paths to the system folders
        :ivar user: :any:`UserObject`, owner of the disk
        :ivar revision: `int`, current revision of Yandex.Disk
    """

    def __init__(self, disk_info=None):
        YaDiskObject.__init__(self, {"max_file_size":  int,
                                     "total_space":    int,
                                     "trash_size":     int,
                                     "is_paid":        bool,
                                     "used_space":     int,
                                     "system_folders": SystemFoldersObject,
                                     "user":           UserObject,
                                     "revision":       int})

        self.import_fields(disk_info)

class SystemFoldersObject(YaDiskObject):
    """
        Object, containing paths to system folders.

        :param system_folders: `dict` or `None`

        :ivar odnoklassniki: `str`, path to the Odnoklassniki folder
        :ivar google: `str`, path to the Google+ folder
        :ivar instagram: `str`, path to the Instagram folder
        :ivar vkontakte: `str`, path to the VKontakte folder
        :ivar mailru: `str`, path to the My World folder
        :ivar facebook: `str`, path to the Facebook folder
        :ivar social: `str`, path to the social networks folder
        :ivar screenshots: `str`, path to the screenshot folder
        :ivar photostream: `str`, path to the camera folder
    """

    def __init__(self, system_folders=None):
        YaDiskObject.__init__(self, {"odnoklassniki": str,
                                     "google":        str,
                                     "instagram":     str,
                                     "vkontakte":     str,
                                     "mailru":        str,
                                     "downloads":     str,
                                     "applications":  str,
                                     "facebook":      str,
                                     "social":        str,
                                     "screenshots":   str,
                                     "photostream":   str})

        self.import_fields(system_folders)

class UserObject(YaDiskObject):
    """
        User object.

        :param user: `dict` or `None`

        :ivar login: `str`, user's login
        :ivar display_name: `str`, user's display name
        :ivar uid: `str`, user's UID
    """

    def __init__(self, user=None):
        YaDiskObject.__init__(self, {"login":        str,
                                     "display_name": str,
                                     "uid":          str})

        self.import_fields(user)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__all__ = ["DEFAULT_TIMEOUT", "DEFAULT_N_RETRIES", "DEFAULT_UPLOAD_TIMEOUT",
           "DEFAULT_UPLOAD_RETRY_INTERVAL"]

# `tuple` of 2 numbers (`int` or float`), default timeout for requests.
# First number is the connect timeout, the second one is the read timeout.
DEFAULT_TIMEOUT = (10.0, 15.0)

# `int`, default number of retries
DEFAULT_N_RETRIES = 3

# `float`, default retry interval
DEFAULT_RETRY_INTERVAL = 0.0

# Analogous to `DEFAULT_TIMEOUT` but for `upload` function
DEFAULT_UPLOAD_TIMEOUT = DEFAULT_TIMEOUT

# Analogous to `DEFAULT_RETRY_INTERVAL` but for `upload` function
DEFAULT_UPLOAD_RETRY_INTERVAL = 0.0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__all__ = ["YaDiskObject"]

class YaDiskObject(object):
    """
        Base class for all objects mirroring the ones returned by Yandex.Disk REST API.
        It must have a fixed number of fields, each field must have a type.
        It also supports subscripting and access of fields through the . operator.

        :param field_types: `dict` or `None`
    """

    def __init__(self, field_types=None):
        if field_types is None:
            field_types = {}

        self.FIELD_TYPES = {}
        self.FIELDS = {}
        self.ALIASES = {}
        self.set_field_types(field_types)

    def set_field_types(self, field_types):
        """
            Set the field types of the object

            :param field_types: `dict`, where keys are the field names and values are types (or factories)
        """

        self.FIELD_TYPES = field_types

        for field in field_types.keys():
            self[field] = None

    def set_field_type(self, field, type):
        """
            Set field type.

            :param field: `str`
            :param type: type or factory
        """

        self.FIELD_TYPES[field] = type
        self[field] = None

    def set_alias(self, alias, name):
        """
            Set an alias.

            :param alias: `str`, alias to add
            :param name: `str`, field name
        """

        self.ALIASES[alias] = name

    def remove_alias(self, alias):
        """
            Remove an alias.

            :param alias: `str`
        """

        self.ALIASES.pop(alias)

    def remove_field(self, field):
        """
            Remove field.

            :param field: `str`
        """

        self.FIELDS.pop(field)
        self.FIELD_TYPES.pop(field)

    def import_fields(self, source_dict):
        """
            Set all the fields of the object to the values in `source_dict`.
            All the other fields are ignored

            :param source_dict: `dict` or `None` (nothing will be done in that case)
        """

        if source_dict is not None:
            for field in self.FIELDS:
                try:
                    self[field] = source_dict[field]
                except KeyError:
                    pass

            for alias, field in self.ALIASES.items():
                try:
                    self[field] = source_dict[alias]
                except KeyError:
                    pass

    def __setattr__(self, attr, value):
        if attr in ("FIELDS", "FIELD_TYPES", "ALIASES"):
            self.__dict__[attr] = value
            return

        attr = self.ALIASES.get(attr, attr)

        if attr not in self.FIELD_TYPES:
            raise AttributeError("Unknown attribute: %r" % (attr,))

        datatype = self.FIELD_TYPES[attr]
        self.FIELDS[attr] = datatype(value) if value is not None else None

    def __getattr__(self, attr):
        if attr in ("FIELDS", "FIELD_TYPES"):
            return self.__dict__[attr]

        attr = self.ALIASES.get(attr, attr)

        if attr not in self.FIELD_TYPES:
            raise AttributeError("Unknown attribute: %r" % (attr,))

        return self.FIELDS[attr]

    def __getitem__(self, key):
        return self.FIELDS[key]

    def __setitem__(self, key, value):
        self.__setattr__(key, value)

    def __delitem__(self, key):
        del self.FIELDS[key]

    def __iter__(self):
        return iter(self.FIELDS)

    def __len__(self):
        return len(self.FIELDS)

    def __repr__(self):
        return "<%s%r>" % (self.__class__.__name__, self.FIELDS)
